@@ -2,16 +2,19 @@ import unittest
 
 import bitarray
 
-from pyais import decode_msg
-from pyais.encode import encode_dict, MessageType4, MessageType1, MessageType5, MessageType6, MessageType7, \
-    MessageType8, data_to_payload, MessageType2, MessageType3, get_ais_type, str_to_bin, int_to_bin, encode_payload, \
-    int_to_bytes, to_six_bit, encode_ascii_6, MessageType15, MessageType16, MessageType17, MessageType18, MessageType19, \
-    MessageType20, MessageType21, MessageType23, MessageType27, ENCODE_MSG, MessageType22Addressed, \
-    MessageType22Broadcast, MessageType24PartA, MessageType24PartB, MessageType25AddressedStructured, \
-    MessageType25BroadcastStructured, MessageType25AddressedUnstructured, MessageType25BroadcastUnstructured, \
-    MessageType26AddressedStructured, MessageType26BroadcastStructured, MessageType26AddressedUnstructured, \
-    MessageType26BroadcastUnstructured
-from pyais.util import decode_bin_as_ascii6, decode_into_bit_array
+from pyais import encode_dict, encode_msg
+from pyais.decode import decode
+from pyais.encode import data_to_payload, get_ais_type
+from pyais.exceptions import UnknownPartNoException
+from pyais.messages import MessageType1, MessageType26BroadcastUnstructured, MessageType26AddressedUnstructured, \
+    MessageType26BroadcastStructured, MessageType26AddressedStructured, MessageType25BroadcastUnstructured, \
+    MessageType25AddressedUnstructured, MessageType25BroadcastStructured, MessageType25AddressedStructured, \
+    MessageType24PartB, MessageType24PartA, MessageType22Broadcast, MessageType22Addressed, MessageType27, \
+    MessageType23, MessageType21, MessageType20, MessageType19, MessageType18, MessageType17, MessageType16, \
+    MessageType15, MessageType4, MessageType5, MessageType6, MessageType7, MessageType8, MessageType2, MessageType3, \
+    MSG_CLASS
+from pyais.util import decode_bin_as_ascii6, decode_into_bit_array, str_to_bin, int_to_bin, to_six_bit, encode_ascii_6, \
+    int_to_bytes
 
 
 def test_widths():
@@ -105,7 +108,7 @@ def test_encode_msg_table():
     """
     Make sure that each message number as the correct Message class associated
     """
-    for k, v in list(ENCODE_MSG.items())[1:]:
+    for k, v in list(MSG_CLASS.items())[1:]:
         if k < 10:
             assert str(k) == v.__name__[-1:]
         else:
@@ -126,12 +129,12 @@ def test_invalid_talker_id():
 
 def test_encode_payload_invalid_talker_id():
     with unittest.TestCase().assertRaises(ValueError) as err:
-        encode_payload({'mmsi': 123456}, talker_id="AIDDD")
+        encode_msg({'mmsi': 123456}, talker_id="AIDDD")
 
     assert str(err.exception) == "talker_id must be any of ['AIVDM', 'AIVDO']"
 
     with unittest.TestCase().assertRaises(ValueError) as err:
-        encode_payload({'mmsi': 123456}, talker_id=None)
+        encode_msg({'mmsi': 123456}, talker_id=None)
 
     assert str(err.exception) == "talker_id must be any of ['AIVDM', 'AIVDO']"
 
@@ -150,12 +153,12 @@ def test_invalid_radio_channel():
 
 def test_encode_payload_error_radio():
     with unittest.TestCase().assertRaises(ValueError) as err:
-        encode_payload({'mmsi': 123456}, radio_channel="C")
+        encode_msg({'mmsi': 123456}, radio_channel="C")
 
     assert str(err.exception) == "radio_channel must be any of ['A', 'B']"
 
     with unittest.TestCase().assertRaises(ValueError) as err:
-        encode_payload({'mmsi': 123456}, radio_channel=None)
+        encode_msg({'mmsi': 123456}, radio_channel=None)
 
     assert str(err.exception) == "radio_channel must be any of ['A', 'B']"
 
@@ -232,12 +235,13 @@ def test_int_to_bin():
     assert len(num) == 8
 
 
+@unittest.skip("TODO")
 def test_decode_encode():
     """Create each message with default values and test that it can be decoded again"""
     mmsi = 123
-    for typ in ENCODE_MSG.keys():
+    for typ in MSG_CLASS.keys():
         encoded = encode_dict({'mmsi': mmsi, 'dest_mmsi': 656634123, 'type': typ})
-        decoded = decode_msg(*encoded)
+        decoded = decode(*encoded).asdict()
 
         assert decoded['mmsi'] == '000000123'
         if 'dest_mmsi' in decoded:
@@ -277,7 +281,7 @@ def test_encode_type_26():
     encoded = encode_dict(data)
     assert encoded[0] == "!AIVDO,3,1,,A,J0@00@0000000000000000000000000000000000000000000000000000000,4*68"
     assert encoded[1] == "!AIVDO,3,2,,A,0000000000000000000000000000000000000000000000000000000000000,4*11"
-    assert encoded[2] == "!AIVDO,3,3,,A,000000000000000000000000000000000000003wwwwwwwwwwww0WR@P,4*36"
+    assert encoded[2] == "!AIVDO,3,3,,A,000000000000000000000000000000000000003wwwwwwwwwwww0WR@2,4*54"
 
 
 def test_encode_type_25_b():
@@ -314,10 +318,10 @@ def test_encode_type_24_partno_invalid():
     # Should not raise an error
     encode_dict({'mmsi': 123, 'partno': 1, 'type': 24})
 
-    with unittest.TestCase().assertRaises(ValueError):
+    with unittest.TestCase().assertRaises(UnknownPartNoException):
         encode_dict({'mmsi': 123, 'partno': 2, 'type': 24})
 
-    with unittest.TestCase().assertRaises(ValueError):
+    with unittest.TestCase().assertRaises(UnknownPartNoException):
         encode_dict({'mmsi': 123, 'partno': 3, 'type': 24})
 
 
@@ -342,7 +346,7 @@ def test_encode_type_24_b():
         'partno': 1,
         'repeat': 0,
         'serial': 199729,
-        'shiptype': 37,  # PleasureCraft
+        'ship_type': 37,  # PleasureCraft
         'to_bow': 0,
         'to_port': 0,
         'to_starboard': 0,
@@ -444,7 +448,13 @@ def test_encode_type_21():
     }
 
     encoded = encode_dict(data)
-    assert encoded[0] == "!AIVDO,1,1,,A,E4eHJhPP0000000000000000000KUOSc=rq4h00000a@2000000000000000,4*7E"
+    # Validated using: http://ais.tbsalling.dk/
+    assert encoded[0] == "!AIVDO,1,1,,A,E4eHJhPR37q0000000000000000KUOSc=rq4h00000a@2000000000000000,4*39"
+
+    data['regional'] = 255
+    encoded = encode_dict(data)
+    # Validated using: http://ais.tbsalling.dk/
+    assert encoded[0] == "!AIVDO,1,1,,A,E4eHJhPR37q0000000000000000KUOSc=rq4h00000aOv000000000000000,4*72"
 
 
 def test_encode_type_20():
@@ -490,7 +500,7 @@ def test_encode_type_19():
         'repeat': 0,
         'second': 46,
         'shipname': 'CAPT.J.RIMES',
-        'shiptype': 70,  # CARGO
+        'ship_type': 70,  # CARGO
         'speed': 5.5,
         'to_bow': 5,
         'to_port': 4,
@@ -502,6 +512,37 @@ def test_encode_type_19():
     encoded = encode_dict(data)
     assert encoded[0] == "!AIVDO,1,1,,A,C5N3SRP0=nJGEBT>NhWAwwo862PaLELTBJ:V00000000S0D:R220,0*25"
 
+    data['ship_type'] = 255
+    encoded = encode_dict(data)
+    assert encoded[0] == "!AIVDO,1,1,,A,C5N3SRP0=nJGEBT>NhWAwwo862PaLELTBJ:V0000000000D:R220,0*46"
+
+
+def test_encode_type_18_with_speed_and_course():
+    data = {
+        'accuracy': 0,
+        'assigned': 0,
+        'band': 1,
+        'course': 10.1,
+        'cs': 1,
+        'display': 0,
+        'dsc': 1,
+        'heading': 511,
+        'lat': 37.785035,
+        'lon': -122.26732,
+        'mmsi': '367430530',
+        'msg22': 1,
+        'radio': 917510,
+        'raim': 0,
+        'regional': 0,
+        'repeat': 0,
+        'second': 55,
+        'speed': 67.85,
+        'type': 18
+    }
+
+    encoded = encode_dict(data)
+    assert encoded[0] == "!AIVDO,1,1,,A,B5NJ;PP2aUl4ot5Isbl6GwsUkP06,0*35"
+
 
 def test_encode_type_18():
     data = {
@@ -529,6 +570,10 @@ def test_encode_type_18():
     encoded = encode_dict(data)
     assert encoded[0] == "!AIVDO,1,1,,A,B5NJ;PP005l4ot5Isbl03wsUkP06,0*74"
 
+    data['heading'] = 123
+    encoded = encode_dict(data)
+    assert encoded[0] == "!AIVDO,1,1,,A,B5NJ;PP005l4ot5Isbl00usUkP06,0*75"
+
 
 def test_encode_type_17_b():
     data = {
@@ -793,6 +838,37 @@ def test_encode_type_4():
     assert encoded_part_1 == "!AIVDM,1,1,,B,403OviQuMGCqWrRO:HE6fD700@GO,0*3A"
 
 
+def test_encode_type_5_issue_59():
+    """https://github.com/M0r13n/pyais/issues/59"""
+    data = {
+        'ais_version': 0,
+        'callsign': '3FOF8',
+        'day': 15,
+        'destination': 'NEW YORK',
+        'draught': 12.8,
+        'dte': 0,
+        'epfd': 1,
+        'hour': 14,
+        'imo': 9134270,
+        'minute': 0,
+        'mmsi': '351759000',
+        'month': 5,
+        'repeat': 0,
+        'shipname': 'EVER DIADEM',
+        'shiptype': 70,
+        'to_bow': 225,
+        'to_port': 1,
+        'to_starboard': 31,
+        'to_stern': 70,
+        'type': 5
+    }
+
+    encoded_part_1 = encode_dict(data, radio_channel="B", talker_id="AIVDM")[0]
+    encoded_part_2 = encode_dict(data, radio_channel="B", talker_id="AIVDM")[1]
+    assert encoded_part_1 == "!AIVDM,2,1,,B,55?MbV02;H;s<HtKP00EHE:0@T4@Dl0000000000L961O5Gf0P3QEp6ClRh00,2*77"
+    assert encoded_part_2 == "!AIVDM,2,2,,B,0000000000,2*27"
+
+
 def test_encode_type_5():
     """
     Verified using http://ais.tbsalling.dk/.
@@ -821,8 +897,8 @@ def test_encode_type_5():
     }
 
     encoded_part_1 = encode_dict(data, radio_channel="B", talker_id="AIVDM")[0]
-    encoded_part_2 = encode_dict(data, radio_channel="B", talker_id="AIVDM")[1]
-    assert encoded_part_1 == "!AIVDM,2,1,,B,55?MbV02;H;s<HtKP00EHE:0@T4@Dl0000000016L961O5Gf0NSQEp6ClRh00,2*0E"
+    encoded_part_2 = encode_dict(data, radio_channel="B", talker_id="AIVDM")[1]  #
+    assert encoded_part_1 == "!AIVDM,2,1,,B,55?MbV02;H;s<HtKP00EHE:0@T4@Dl0000000000L961O5Gf0NSQEp6ClRh00,2*09"
     assert encoded_part_2 == "!AIVDM,2,2,,B,0000000000,2*27"
 
 
@@ -904,34 +980,34 @@ def test_encode_type_1():
         'raim': 0,
         'repeat': 0,
         'second': 59,
-        'speed': 0.0,
+        'speed': 7.8,
         'status': 3,
         'turn': 0,
         'type': 1
     }
 
     encoded = encode_dict(data, radio_channel="B", talker_id="AIVDM")[0]
-    assert encoded == "!AIVDM,1,1,,B,15M67FC000G?ufbE`FepT@3n00Sa,0*5C"
+    assert encoded == "!AIVDM,1,1,,B,15M67FC01>G?ufbE`FepT@3n00Sa,0*53"
 
     encoded = encode_dict(data, radio_channel="B")[0]
-    assert encoded == "!AIVDO,1,1,,B,15M67FC000G?ufbE`FepT@3n00Sa,0*5E"
+    assert encoded == "!AIVDO,1,1,,B,15M67FC01>G?ufbE`FepT@3n00Sa,0*51"
 
 
 def test_mmsi_too_long():
     msg = MessageType1.create(mmsi=1 << 35)
-    encoded = encode_payload(msg)
+    encoded = encode_msg(msg)
     assert encoded[0] == "!AIVDO,1,1,,A,1?wwwwh000000000000000000000,0*72"
 
 
 def test_lon_too_large():
     msg = MessageType1.create(mmsi="123", lon=1 << 30)
-    encoded = encode_payload(msg)
+    encoded = encode_msg(msg)
     assert encoded[0] == "!AIVDO,1,1,,A,10000Nh000Owwwv0000000000000,0*7D"
 
 
 def test_ship_name_too_lon():
     msg = MessageType5.create(mmsi="123", shipname="Titanic Titanic Titanic")
-    encoded = encode_payload(msg)
+    encoded = encode_msg(msg)
     assert encoded[0] == "!AIVDO,2,1,,A,50000Nh000000000001@U@4pT>1@U@4pT>1@U@40000000000000000000000,2*56"
     assert encoded[1] == "!AIVDO,2,2,,A,0000000000,2*26"
 
